@@ -4,14 +4,18 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use utf8; # filenames can be in utf8
 
+binmode STDERR, ':encoding(UTF-8)';
+binmode STDOUT, ':encoding(UTF-8)';
+
 use File::Temp;
 use File::Find::Rule;
 use IPC::Run;
 use Text::ParseWords;
+use Encode;
 use Data::Roundtrip qw/perl2dump no-unicode-escape-permanently/;
 
 sub	new {
@@ -36,42 +40,63 @@ sub	new {
 		# or <=0 for using FFmpeg's defaults
 		'frame-duration' => 0,
 		# this will be modified during perl Makefile.PL
-		'ffmpeg-executable' => '/usr/local/bin/ffmpeg' # specify fullpath if not in path
+		'ffmpeg-executable' => '/usr/local/bin/ffmpeg', # specify fullpath if not in path
 		# end this will be modified during perl Makefile.PL
 	};
 	bless $self, $class;
 
+	my $ak;
+
 	# sort out the easy params
-	for my $ak (qw/
-		verbosity
+	my $verbos = 0;
+	$ak = 'verbosity';
+	if( exists($params->{$ak}) && defined($verbos=$params->{$ak}) ){ $self->verbosity($verbos) }
+
+	for $ak (qw/
 		output-filename
 		frame-duration
 	/){
 		if( exists($params->{$ak}) && defined($params->{$ak}) ){
-			$self->{$ak} = $params->{$ak}
+			$self->{$ak} = $params->{$ak};
+			if( $verbos > 1 ){ print "${whoami} (via $parent), line ".__LINE__." : parameter '$ak' set to : ".$params->{$ak}."\n"; }
 		}
 	}
 
 	# specify input images as scalar, arrayref or hashref (values)
-	if( exists($params->{'input-images'}) && defined($params->{'input-images'})
-		&& scalar($params->{'input-images'})
+	$ak = 'input-images';
+	if( exists($params->{$ak}) && defined($params->{$ak})
+		&& scalar($params->{$ak})
 	){
-		$self->input_images($params->{'input-images'});
+		$self->input_images($params->{$ak});
+	}
+
+	# input image filenames are read from specified file (one filename per line)
+	$ak = 'input-images-from-file';
+	if( exists($params->{$ak}) && defined($params->{$ak})
+		&& scalar($params->{$ak})
+	){
+		if( ! $self->input_file_with_images($params->{$ak}) ){ print STDERR perl2dump($params->{$ak})."${whoami} (via $parent), line ".__LINE__." : error, failed to load input images from file containing their pathnames: '".$params->{$ak}."'.\n"; return undef }
 	}
 
 	# input images can be specified via a pattern and a search dir
 	# like : 'input-pattern' => ['*.png', '/x/y/searchdir']
-	if( exists($params->{'input-pattern'}) && defined($params->{'input-pattern'}) ){
-		if( ref($params->{'input-pattern'})ne'ARRAY' ){ print STDERR perl2dump($params->{'input-pattern'})."${whoami} (via $parent), line ".__LINE__." : error, the argument to 'input-pattern' must be an ARRAYref of 1 or 2 items: the pattern and optionally the search dir. See above for what was provided.\n"; return undef }
-		if( ! $self->input_pattern($params->{'input-pattern'}) ){ print STDERR perl2dump($params->{'input-pattern'})."${whoami} (via $parent), line ".__LINE__." : error, failed to find input files based on the above pattern and search dir.\n"; return undef }
+	$ak = 'input-pattern';
+	if( exists($params->{$ak}) && defined($params->{$ak}) ){
+		if( ref($params->{$ak})ne'ARRAY' ){ print STDERR perl2dump($params->{$ak})."${whoami} (via $parent), line ".__LINE__." : error, the argument to '$ak' must be an ARRAYref of 1 or 2 items: the pattern and optionally the search dir. See above for what was provided.\n"; return undef }
+		if( ! $self->input_pattern($params->{$ak}) ){ print STDERR perl2dump($params->{$ak})."${whoami} (via $parent), line ".__LINE__." : error, failed to find input files based on the above pattern and search dir.\n"; return undef }
 	}
-	if( exists($params->{'input-patterns'}) && defined($params->{'input-patterns'}) ){
-		if( ref($params->{'input-patterns'})ne'ARRAY' ){ print STDERR perl2dump($params->{'input-patterns'})."${whoami} (via $parent), line ".__LINE__." : error, the argument to 'input-patterns' must be an ARRAYref of one or more ARRAYrefs each of 1 or 2 items: the pattern and optionally the search dir. See above for what it was provided.\n"; return undef }
-		if( ! $self->input_patterns($params->{'input-patterns'}) ){ print STDERR perl2dump($params->{'input-patterns'})."${whoami} (via $parent), line ".__LINE__." : error, failed to find input files based on the above pattern and search dir.\n"; return undef }
+	# or via multiple patterns (an ARRAY of ARRAY patterns, as above)
+	$ak = 'input-patterns';
+	if( exists($params->{$ak}) && defined($params->{$ak}) ){
+		if( ref($params->{$ak})ne'ARRAY' ){ print STDERR perl2dump($params->{$ak})."${whoami} (via $parent), line ".__LINE__." : error, the argument to '$ak' must be an ARRAYref of one or more ARRAYrefs each of 1 or 2 items: the pattern and optionally the search dir. See above for what it was provided.\n"; return undef }
+		if( ! $self->input_patterns($params->{$ak}) ){ print STDERR perl2dump($params->{$ak})."${whoami} (via $parent), line ".__LINE__." : error, failed to find input files based on the above pattern and search dir.\n"; return undef }
 	}
+
 	# specify output filename
-	if( exists($params->{'output-filename'}) && defined($params->{'output-filename'}) ){
-		$self->output_filename($params->{'output-filename'})
+	$ak = 'output-filename';
+	if( exists($params->{$ak}) && defined($params->{$ak}) ){
+		$self->output_filename($params->{$ak});
+		if( $verbos > 1 ){ print "${whoami} (via $parent), line ".__LINE__." : parameter '$ak' set to : ".$params->{$ak}."\n"; }
 	}
 	# any extra ffmpeg params?
 	# these are cmdline options to FFmpeg and must be
@@ -79,9 +104,12 @@ sub	new {
 	# a single array item, for example
 	# ['-i', 'inputfile', '-o', 'out', '-p', '1', '2']
 	# note that above -p is used as in -p 1 2
-	if( exists($params->{'ffmpeg-extra-params'}) && defined($params->{'ffmpeg-extra-params'}) ){
-		if( ! defined($self->ffmpeg_extra_params($params->{'ffmpeg-extra-params'})) ){ print STDERR perl2dump($params->{'ffmpeg-extra-params'})."${whoami} (via $parent), line ".__LINE__." : error, failed to parse/verify the above params to ffmpeg via '--ffmpeg-extra-params'.\n"; return undef }
+	$ak = 'ffmpeg-extra-params';
+	if( exists($params->{$ak}) && defined($params->{$ak}) ){
+		if( ! defined($self->ffmpeg_extra_params($params->{$ak})) ){ print STDERR perl2dump($params->{$ak})."${whoami} (via $parent), line ".__LINE__." : error, failed to parse/verify the above params to ffmpeg via '--ffmpeg-extra-params'.\n"; return undef }
+		if( $verbos > 1 ){ print "${whoami} (via $parent), line ".__LINE__." : parameter '$ak' set to : ".$params->{$ak}."\n"; }
 	}
+
 	return $self
 }
 # it spawns ffmpeg as external command via IPC::Run::run(@cmd)
@@ -101,14 +129,15 @@ sub	make_animation {
 	my ($in, $out, $err);
 	my $ret = IPC::Run::run($cmdline, \$in, \$out, \$err);
 	if( ! $ret ){ 
-		print STDERR "$out\n$err\n${whoami} (via $parent), line ".__LINE__." : error, executing this command has failed (the list of input files is in '$tmpfile'):\n  ".join(' ', @$cmdline)."\n";
+		print STDERR "$out\n$err\n${whoami} (via $parent), line ".__LINE__." : error, executing this command has failed (the list of input files is in '$tmpfile', you may delete it when you are done):\n  ".join(' ', @$cmdline)."\n";
 		return 0
 	}
 	if( $verbos > 0 ){
 		if( $verbos > 1 ){ print $out."\n" }
-		print "${whoami} (via $parent), line ".__LINE__." : done, success. Output is in '".$self->output_filename()."'.\n"
+		print "${whoami} (via $parent), line ".__LINE__." : done, success. Output animation of ".$self->num_input_images()." input images is in '".$self->output_filename()."'.\n"
 	}
 	unlink($tmpfile);
+
 	return 1;
 }
 sub	_build_ffmpeg_cmdline {
@@ -120,19 +149,19 @@ sub	_build_ffmpeg_cmdline {
 	if( $self->num_input_images() == 0 ){ print STDERR "${whoami} (via $parent), line ".__LINE__." : error, no input images in store.\n"; return undef }
 	if( ! defined $self->output_filename() ){ print STDERR "${whoami} (via $parent), line ".__LINE__." : error, no output filename specified.\n"; return undef }
 
-	# get a tmp file
-	my ($fh, $tmpfile) = File::Temp::tempfile();
-	my $duration_str = $self->frame_duration() > 0 ? "duration ".$self->frame_duration()."\n" : "";
-	for (@{$self->{'input-images'}}){
-		print $fh "file '".$_
-			 ."'\n${duration_str}"
-		;
-	}
-	close $fh;
-
 	my $execu = $self->ffmpeg_executable();
 	if( ! defined $execu ){ print STDERR "$whoami() (via $parent), line ".__LINE__." : error, the path to the ffmpeg executable is undefined. That means that the external, 3rd-party dependency 'ffmpeg' was not located during installation. First you need to install ffmpeg. Then re-install current module. Note that if ffmpeg is installed in a non-standard location you need to specify that location (i.e. the full-path to the ffmpeg executable) at installation of this module by setting the environment variable 'AUTOMATE_ANIMATE_FFMPEG_PATH'. See the documentation (https://metacpan.org/pod/Automate::Animate::FFmpeg#INSTALLATION) for further details.\n"; return undef }
 
+	# get a tmp file - only drawback is it is in current working dir
+	# but on the other hand we do not want caller to delete a directory!
+	my ($fh, $tmpfile) = File::Temp::tempfile('XXXXXXXXXXXX', SUFFIX => '.txt');
+	my $duration_str = $self->frame_duration() > 0 ? "duration ".$self->frame_duration()."\n" : "";
+	binmode $fh, ':encoding(UTF-8)';
+	print $fh "file '".$_."'\n"
+			  .${duration_str}
+	for @{$self->{'input-images'}};
+	close $fh;
+
 	my @cmdline = (
 		$execu,
 		# the extra params to FFmpeg is just a command-line args style
@@ -170,6 +199,8 @@ sub	frame_duration {
 	return $m
 }
 
+# note: when setting an output filename, make sure you
+# specify the extension and it does make sense to FFmpeg (e.g. mp4)
 sub	output_filename {
 	my $self = $_[0];
 	my $outfile = $_[1];
@@ -225,27 +256,34 @@ sub	clear_input_images { $#{ $_[0]->{'input-images'} } = -1 }
 # make sure that the order you expect is what you get during the pattern materialisation
 # the search dir is optional, default is Cwd::cwd
 sub	input_pattern {
-	my ($self, $_pattern, $indir) = @_;
+	my ($self, $params) = @_;
 	my $parent = ( caller(1) )[3] || "N/A";
 	my $whoami = ( caller(0) )[3];
 	my $verbos = $self->verbosity();
+	my ($_pattern, $indir) = @$params;
 	if( ! defined $indir ){
 		$indir = Cwd::cwd;
 		if( $verbos > 0 ){ print STDERR "${whoami} (via $parent), line ".__LINE__." : warning, no search dir was specified and using current dir '$indir'.\n" }
 	}
 	my $pattern;
+	# allows for /pattern/modifiers
 	if( $_pattern =~ m!^regex\(/(.*?)/(.*?)\)$! ){
 		# see https://www.perlmonks.org/?node_id=1210675
 		my $pa = $1; my $mo = $2;
 		if( $mo!~/^[msixpodualn]+$/ ){ print STDERR "${whoami} (via $parent), line ".__LINE__." : error, illegal modifiers ($mo) to the specified regex detected.\n"; return 0 }
+		# the modifiers are entered as (?...) before the regex pattern
 		$pattern = qr/(?${mo})${pa}/;
 	} else { $pattern = $_pattern }
+	if( $verbos > 1 ){ print "${whoami} (via $parent), line ".__LINE__." : searching under dir '$indir' with pattern '".$pattern."' ...\n" }
+
 	if( ! defined $self->input_images([
-		File::Find::Rule
+		# this little piglet does not support unicode or rather readdir needs some patching
+		map { Encode::decode_utf8($_) } File::Find::Rule
 			->file()
 			->name($pattern)
 			->in($indir)
 	]) ){ print STDERR "${whoami} (via $parent), line ".__LINE__." : error, call to input_images() has failed.\n"; return 0 }
+
 	return 1 # success
 }
 # This adds many patterns:
@@ -257,8 +295,10 @@ sub	input_patterns {
 	my $whoami = ( caller(0) )[3];
 	my $verbos = $self->verbosity();
 	for my $as (@$specs){
-		if( scalar(@$as)==0 ){ print STDERR perl2dump($as)."${whoami} (via $parent), line ".__LINE__." : error, the spec must contain at least a pattern and optionally a search-dir as an array, see above.\n"; return 0; }
-		if( ! $self->input_pattern(@$as) ){ print STDERR "${whoami} (via $parent), line ".__LINE__." : error, call to input_pattern() has failed for this spec: @$as\n"; return 0 }
+		if( (scalar(@$as)==0)
+		 || (scalar(@$as)>2)
+		){ print STDERR perl2dump($as)."${whoami} (via $parent), line ".__LINE__." : error, the spec must contain at least a pattern and optionally a search-dir as an array, see above.\n"; return 0; }
+		if( ! $self->input_pattern($as) ){ print STDERR "${whoami} (via $parent), line ".__LINE__." : error, call to input_pattern() has failed for this spec: @$as\n"; return 0 }
 	}
 	return 1 # success
 }
@@ -274,38 +314,43 @@ sub	input_images {
 	my $whoami = ( caller(0) )[3];
 	my $verbos = $self->verbosity();
 	if( $verbos > 0 ){
-		if( $verbos > 1 ){ print STDOUT perl2dump($m)."${whoami} (via $parent), line ".__LINE__." : called with above parameter ...\n" }
+		if( $verbos > 1 ){ print STDOUT perl2dump($m)."${whoami} (via $parent), line ".__LINE__." : called with input images as shown above ...\n" }
 		else { print STDOUT "${whoami} (via $parent), line ".__LINE__." : called ...\n" }
 	}
+	# NOTE: Cwd::abs_path() messes up on unicode filenames and requires Encode::decode_utf8()
+	# but there is also Cwd::utf8 to consider sometime...
 	my $rf = ref $m;
 	if( $rf eq 'ARRAY' ){
 		for my $af (@$m){
 			if( ! -e $af ){ print STDERR "${whoami} (via $parent), line ".__LINE__." : warning/1, input image '$af' does not exist on disk and will be ignored.\n"; next }
-			push @{$self->{'input-images'}}, Cwd::abs_path($af)
+			push @{$self->{'input-images'}}, Encode::decode_utf8(Cwd::abs_path($af));
 		}
 	} elsif( $rf eq 'HASH' ){
 		for my $af (values %$m){
 			if( ! -e $af ){ print STDERR "${whoami} (via $parent), line ".__LINE__." : warning/2, input image '$af' does not exist on disk and will be ignored.\n"; next }
-			push @{$self->{'input-images'}}, Cwd::abs_path($af)
+			push @{$self->{'input-images'}}, Encode::decode_utf8(Cwd::abs_path($af));
 		}		
 	} elsif( $rf eq '' ){
 		if( ! -e $m ){ print STDERR "${whoami} (via $parent), line ".__LINE__." : warning/3, input image '$m' does not exist on disk and will be ignored.\n"; }
-		else { push @{$self->{'input-images'}}, Cwd::abs_path($m) }
+		else { push @{$self->{'input-images'}}, Encode::decode_utf8(Cwd::abs_path($m)) }
 	} else { print STDERR "${whoami} (via $parent), line ".__LINE__." : error, input can be an arrayref of input image filenames, a hashref whose values are filenames or a single filename in the form of a scalar."; return undef }
-	if( $verbos > 1 ){ print STDOUT perl2dump($self->{'input-images'})."${whoami} (via $parent), line ".__LINE__." : called and added some images, above is a list of all known images.\n" }
+	if( $verbos > 1 ){ print STDOUT perl2dump($self->{'input-images'})."${whoami} (via $parent), line ".__LINE__." : called and added some images, above is a list of all input images to make up the animation.\n" }
 	return $self->{'input-images'}
 }
 
 # that's the end, pod now starts
+
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Automate::Animate::FFmpeg - Create animation from a sequence of images using FFmpeg
 
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 
 =head1 SYNOPSIS
@@ -318,27 +363,47 @@ FFmpeg binaries must already be installed in your system.
 
     use Automate::Animate::FFmpeg;
     my $aaFFobj = Automate::Animate::FFmpeg->new({
-      # specify input images in any of these 3 ways:
+      # specify input images in any of these 4 ways or a combination:
+      # 1) by specifying each input image (in the order to appear)
+      #    in an ARRAYref
       'input-images' => [
-        'im1.png',
-        'im2.png',
+        '/xyz/abc/im1.png',
+        '/xyz/abc/im2.png',
         ...
       ],
+      # 2) by specifying an input pattern (glob or regex)
+      #    and optional search path
       'input-pattern' => ['*.png', './'],
+      # 3) by specifying an ARRAY of input patterns
+      #    (see above)
+      'input-patterns' => [
+          ['*.tiff'],
+          # specify a regex to filter-in all files under search dir
+	  # NOTE: observe the escaping rules for each quotation method you use
+          [qw!regex(/photos2023-.+?\\.png/i)!, 'abc/xyz'],
+      ],
+      # 4) by specifying a file which contains filenames
+      #    of the input images.
       'input-images-from-file' => 'file-containing-a-list-of-pathnames-to-images.txt',
-      # optionally specify the duration of each frame
+
+      # optionally specify the duration of each frame=image
       'frame-duration' => 5.3, # seconds
+
       'output-filename' => 'out.mp4',
     });
+    # no animation yet!
 
     # options can be set after construction as well:
 
     # optionally add some extra params to FFmpeg as an arrayref
     $aaFF->ffmpeg_extra_params(['-x', 'abc', '-y', 1, 2, 3]);
+
     # you can also add images here, order is important
     $aaFF->input_images(['img1.png', 'img2.png']) or die;
+
     # or add images via a search pattern and optional search dir
     $aaFF->input_pattern(['*.png', './']);
+
     # or add images via multiple search patterns
     $aaFF->input_patterns([
 	['*.png', './'],
@@ -428,8 +493,6 @@ Thank you L<Discipulus|https://www.perlmonks.org/?node_id=174111>.
 
 =item * C<frame-duration> : set the duration of each frame (i.e. each input image) in the animation in (fractional) seconds.
 
-=item * C<ffmpeg-executable> : set the path to the C<ffmpeg> executable.
-
 =item * C<qw/verbosity> : set the verbosity, 0 being mute.
 
 =back
@@ -508,6 +571,24 @@ C<input_pattern()> can be combined with C<input_patterns()>
 and C<input_images()>. The input images list will increase
 in the order they are called.
 
+B<Caveat>: the regex is parsed, compiled and passed on to L<File::Find::Rule>.
+Escaping of special characters (e.g. the backslash) may be required.
+
+B<Caveat>: the order of the matched input images is entirely up
+to L<File::Find::Rule>. There may be unexpected results
+when filenames contain unicode characters. Consider
+these orderings for example:
+
+=over 2
+
+=item * C<blue.png, κίτρινο.png, red.png>,
+
+=item * C<blue.png, γάμμα.png, κίτρινο.png, red.png>,
+
+=item * C<blue.png, κίτρινο.png, γαμμα.png red.png>,
+
+=back
+
 It returns 1 on success or 0 on failure.
 
 =head2 C<input_patterns($m)>
@@ -531,6 +612,10 @@ It returns 1 on success or 0 on failure.
 
 Sets or gets the output filename of the animation.
 
+When setting an output filename, make sure you
+specify its extension and it does make sense to FFmpeg (e.g. mp4).
+
+
 =head2 C<input_file_with_images($m)>
 
   $aaFF->input_file_with_images($m) or die "failed";
@@ -579,36 +664,55 @@ Frame duration is the time that each frame(=image) appears
 in the produced animation.
 
 
-=head1 SCRIPTS 
+=head1 SCRIPTS
 
 A script for making animations from input images using C<ffmpeg>
 is provided: C<automate-animate-ffmpeg.pl>.
 
 It accepts the following options:
 
-    --input-image I [--input-image I2 ...] : specify the full path of an 
+    --input-image I [--input-image I2 ...] : specify the full path of an
     image to be added to the animation. Multiple images are expected.
 
       OR
 
-    --input-images-from-file F [--input-images-from-file F2 ...] : 
-    specify a file which contains a list of input images to be animated, 
-    each on its own line. Multiple images are expected.
+    --input-images-from-file F [--input-images-from-file F2 ...] :
+    specify a file which contains a list of input images to be
+    animated, each on its own line. Multiple images are expected.
 
       OR
 
-    --input-pattern P D : specify a pattern AND a search dir to select 
-    the files from disk. This pattern must be accepted by 
-    File::Find::Rule::name().
+    --input-pattern/-p P [D] : specify a pattern and optional search
+    dir to select the files from disk. This pattern must be accepted
+    by File::Find::Rule::name(). If search dir is not specified,
+    the current working dir will be used.
 
-    --output-filename O : the filename of the output animation.
+    --output-filename/-o O : the filename of the output animation.
 
-    [--frame-duration SECONDS : specify the duration of each frame=input 
-    image in (fractional) seconds.]
+    [--frame-duration/-d SECONDS : specify the duration of each
+    frame=input image in (fractional) seconds.]
 
-    [--verbosity N : specify verbosity. Zero being the mute. Default is 
-    0.]
+    [--verbosity/-V N : specify verbosity. Zero being the mute.
+    Default is 0.]
 
+As an example,
+
+    automate-animate-ffmpeg.pl \
+       --input-pattern '*.png' 't/t-data/images' \
+       --output-filename out.mp4 \
+       --frame-duration 3.5
+
+    # or
+
+    automate-animate-ffmpeg.pl \
+       --input-pattern 'regex(/.+?.png/i)' \
+       --output-filename out.mp4 \
+       --frame-duration 3.5
+
+=head2 UNICODE FILENAMES
+
+Unicode filenames are supported ... I think. Please report
+any problems.
 
 
 =head1 AUTHOR
