@@ -3,10 +3,11 @@
 use strict;
 use warnings;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use Getopt::Long;
 use Text::ParseWords;
+use Encode;
 use Automate::Animate::FFmpeg;
 
 use Data::Roundtrip qw/perl2dump no-unicode-escape-permanently/;
@@ -22,19 +23,23 @@ my $constructor_params = {
 
 my @inimages_from_file;
 
+sub enc { Encode::decode_utf8($_[0]) }
+
 my $waiting_for_second_arg = undef;
 if( ! Getopt::Long::GetOptions(
 	'input-image|i=s' => sub { 
 		if( ! defined $constructor_params->{'input-images'} ){ $constructor_params->{'input-images'} = [ $_[1] ] }
-		else { push @{ $constructor_params->{'input-images'} }, $_[1] }
+		else { push @{ $constructor_params->{'input-images'} },enc($_[1]) }
 	},
 	# this requires both a pattern and a search-dir
 	'input-pattern|p=s{1,2}' => sub {
 		# when we have an option with 2+ args, it comes here twice
 		# once with 1st arg, second time with 2nd arg, etc.
 		my ($k, $v);
-		if( defined $waiting_for_second_arg ){ $k = $waiting_for_second_arg; $v = $_[1]; }
-		else { $k = $_[1]; $v = undef }
+		if( defined $waiting_for_second_arg ){
+			$k = $waiting_for_second_arg;
+			$v =enc($_[1]);
+		} else { $k =enc($_[1]); $v = undef }
 		if( ! defined $constructor_params->{$_[0]} ){
 			$constructor_params->{$_[0]} = [ $k  ]
 		} else {
@@ -42,12 +47,12 @@ if( ! Getopt::Long::GetOptions(
 		}
 		$waiting_for_second_arg = defined($waiting_for_second_arg) ? undef : $k;
 	},
-	'input-images-from-file|f=s' => \@inimages_from_file,
-	'output-filename=s' => sub { $constructor_params->{$_[0]} = $_[1] },
-	'frame-duration=s' => sub { $constructor_params->{$_[0]} = $_[1] },
+	'input-images-from-file|f=s' => sub { push @inimages_from_file,enc($_[1]) },
+	'output-filename|o=s' => sub { $constructor_params->{$_[0]} =enc($_[1]) },
+	'frame-duration|f=s' => sub { $constructor_params->{$_[0]} = $_[1] },
 	# we pass extra params to ffmpeg, not as a string but as an array of options.
-	'ffmpeg-extra-params' => sub { $constructor_params->{$_[0]} = $_[1] },
-	'verbosity=i' => sub { $constructor_params->{$_[0]} = $_[1] },
+	'ffmpeg-extra-params|X=s' => sub { $constructor_params->{$_[0]} =enc($_[1]) },
+	'verbosity|V=i' => sub { $constructor_params->{$_[0]} = $_[1] },
 	'help|h|?' => sub { print STDOUT usage($0)."\n"; exit(0); },
 ) ){ print STDERR usage($0) . "\n\n$0 : error, something wrong with command line parameters.\n"; exit(1) }
 
@@ -75,7 +80,7 @@ sub usage {
 	"  OR\n".
 	"--input-images-from-file F [--input-images-from-file F2 ...] : specify a file which contains a list of input images to be animated, each on its own line. Multiple images are expected.\n".
 	"  OR\n".
-	"--input-pattern P D : specify a pattern AND a search dir to select the files from disk. This pattern must be accepted by File::Find::Rule::name().\n".
+	"--input-pattern P [D] : specify a pattern and optional search dir to select the files from disk. This pattern must be accepted by File::Find::Rule::name(). If search dir is not specified, the current working dir will be used.\n".
 
 	"--output-filename O : the filename of the output animation.\n".
 	"[--frame-duration SECONDS : specify the duration of each frame=input image in (fractional) seconds.]\n".
